@@ -1,5 +1,6 @@
 use once_cell::sync::Lazy;
 use semver::Version;
+use sha2::Digest;
 
 use std::{
     ffi::OsString,
@@ -13,7 +14,10 @@ mod error;
 pub use error::SolcVmError;
 
 mod platform;
+pub use platform::platform;
+
 mod releases;
+pub use releases::{all_releases, Releases};
 
 /// Declare path to Solc Version Manager's home directory, "~/.svm" on Unix-based machines.
 pub static SVM_HOME: Lazy<PathBuf> = Lazy::new(|| {
@@ -56,6 +60,13 @@ pub fn use_version(version: &Version) -> Result<(), SolcVmError> {
     Ok(())
 }
 
+/// Unset the global version. This should be done if all versions are removed.
+pub fn unset_global_version() -> Result<(), SolcVmError> {
+    let mut v = fs::File::create(global_version_path().as_path())?;
+    v.write_all("".as_bytes())?;
+    Ok(())
+}
+
 /// Reads the list of Solc versions that have been installed in the machine. The version list is
 /// sorted in ascending order.
 pub fn installed_versions() -> Result<Vec<Version>, SolcVmError> {
@@ -101,9 +112,22 @@ pub async fn install(version: &Version) -> Result<(), SolcVmError> {
         .releases
         .get(version)
         .ok_or(SolcVmError::UnknownVersion)?;
-    let download_url = releases::artifact_url(platform::platform(), artifact.to_string())?;
+    let download_url =
+        releases::artifact_url(platform::platform(), version, artifact.to_string().as_str())?;
 
     let res = reqwest::get(download_url).await?;
+    let binbytes = res.bytes().await?;
+    let mut hasher = sha2::Sha256::new();
+    hasher.update(&binbytes);
+    let cs = &hasher.finalize()[..];
+    let checksum = artifacts
+        .get_checksum(version)
+        .unwrap_or_else(|| panic!("checksum not available: {:?}", version.to_string()));
+
+    // checksum does not match
+    if cs != checksum {
+        return Err(SolcVmError::ChecksumMismatch(version.to_string()));
+    }
 
     let mut dest = {
         setup_version(version.to_string().as_str())?;
@@ -112,8 +136,7 @@ pub async fn install(version: &Version) -> Result<(), SolcVmError> {
         f.set_permissions(Permissions::from_mode(0o777))?;
         f
     };
-
-    let mut content = Cursor::new(res.bytes().await?);
+    let mut content = Cursor::new(binbytes);
     std::io::copy(&mut content, &mut dest)?;
 
     Ok(())
@@ -125,11 +148,19 @@ pub fn remove_version(version: &Version) -> Result<(), SolcVmError> {
     Ok(())
 }
 
-fn setup_home() -> Result<PathBuf, SolcVmError> {
+/// Setup SVM home directory.
+pub fn setup_home() -> Result<PathBuf, SolcVmError> {
+    // create ~/.svm
     let home_dir = SVM_HOME.to_path_buf();
     if !home_dir.as_path().exists() {
         fs::create_dir_all(home_dir.clone())?;
     }
+    // create ~/.svm/.global-version
+    let mut global_version = SVM_HOME.to_path_buf();
+    global_version.push(".global-version");
+    if !global_version.as_path().exists() {
+        fs::File::create(global_version.as_path())?;
+    }
     Ok(home_dir)
 }
 
